@@ -1,5 +1,5 @@
 /* Spamassassin in local_scan by Marc MERLIN <marc_soft@merlins.org> */
-/* $Id: sa-exim.c,v 1.65 2004/08/16 14:37:19 marcmerlin Exp $ */
+/* $Id: sa-exim.c,v 1.70 2005/01/17 08:06:53 marcmerlin Exp $ */
 /*
 
 The inline comments and minidocs were moved to the distribution tarball
@@ -41,10 +41,6 @@ extern uschar *primary_hostname;
 #define RTLD_NOW 0x002
 #endif    
 
-#ifndef SAFEMESGIDCHARS
-#define SAFEMESGIDCHARS "!#%( )*+,-.0123456789:<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[]^_abcdefghijklmnopqrstuvwxyz{|}~";
-#endif
-
 /* Return the verion of the local_scan ABI, if being compiled as a .so */
 int local_scan_version_major(void)
 {
@@ -67,6 +63,11 @@ int local_scan_version(void)
 #endif
 #endif
 
+#ifndef SAFEMESGIDCHARS
+#define SAFEMESGIDCHARS "!#%( )*+,-.0123456789:<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[]^_abcdefghijklmnopqrstuvwxyz{|}~";
+#endif
+
+
 /******************************/
 /* Compile time config values */
 /******************************/
@@ -246,7 +247,7 @@ static int savemail(int readfd, off_t fdstart, char *dir, char *dirvarname,
     /* Let's not worry about you receiving two spams at the same second
      * with the same message ID. If you do, the second one will overwrite
      * the first one */
-    writefd=creat(string_sprintf("%s/new/%s", dir, filename), S_IRUSR|S_IWUSR);
+    writefd=creat(string_sprintf("%s/new/%s", dir, filename), S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH);
     CHECKERR(writefd, string_sprintf("creat %s/new/%s", dir, filename),__LINE__);
 
     /* make the file look like a valid mbox -- idea from dman */
@@ -566,19 +567,24 @@ int local_scan(volatile int fd, uschar **return_text)
     static char *SAerrorsave=NULL;
     static char *SAerrorSavCond="1";
     static int SAtemprejectonerror=0;
-    static float SAteergrube=1<<30;
+    static char *SAteergrube="1048576";
+    static float SAteergrubethreshold;
+    /* This is obsolete, since SAteergrube (now a condition) can do the same */
     static char *SAteergrubecond="1";
     static int SAteergrubetime=900;
     static char *SAteergrubeSavCond="1";
     static char *SAteergrubesave=NULL;
     static int SAteergrubeoverwrite=1;
-    static float SAdevnull=1<<30;
+    static char *SAdevnull="1048576";
+    static float SAdevnullthreshold;
     static char *SAdevnullSavCond="1";
     static char *SAdevnullsave=NULL;
-    static float SApermreject=1<<30;
+    static char *SApermreject="1048576";
+    static float SApermrejectthreshold;
     static char *SApermrejectSavCond="1";
     static char *SApermrejectsave=NULL;
-    static float SAtempreject=1<<30;
+    static char *SAtempreject="1048576";
+    static float SAtemprejectthreshold;
     static char *SAtemprejectSavCond="1";
     static char *SAtemprejectsave=NULL;
     static int SAtemprejectoverwrite=1;
@@ -620,7 +626,7 @@ int local_scan(volatile int fd, uschar **return_text)
     */
 
     /* If you discard a mail with exim ACLs, we get 0 recipients, so let's just
-     * accept the mail, which is bogus either way since it'll get dropped
+     * accept the mail, which won't matter either way since it'll get dropped
      * (thanks to John Horne for reporting this corner case) */
     if (recipients_count == 0) 
     {
@@ -722,19 +728,19 @@ int local_scan(volatile int fd, uschar **return_text)
 	    M_CHECKFORSTR(SAerrorsave);
 	    M_CHECKFORSTR(SAerrorSavCond);
 	    M_CHECKFORVAR(SAtemprejectonerror, "%d");
-	    M_CHECKFORVAR(SAteergrube, "%f");
+	    M_CHECKFORSTR(SAteergrube);
 	    M_CHECKFORSTR(SAteergrubecond);
 	    M_CHECKFORVAR(SAteergrubetime, "%d");
 	    M_CHECKFORSTR(SAteergrubeSavCond);
 	    M_CHECKFORSTR(SAteergrubesave);
 	    M_CHECKFORVAR(SAteergrubeoverwrite, "%d");
-	    M_CHECKFORVAR(SAdevnull, "%f");
+	    M_CHECKFORSTR(SAdevnull);
 	    M_CHECKFORSTR(SAdevnullSavCond);
 	    M_CHECKFORSTR(SAdevnullsave);
-	    M_CHECKFORVAR(SApermreject, "%f");
+	    M_CHECKFORSTR(SApermreject);
 	    M_CHECKFORSTR(SApermrejectsave);
 	    M_CHECKFORSTR(SApermrejectSavCond);
-	    M_CHECKFORVAR(SAtempreject, "%f");
+	    M_CHECKFORSTR(SAtempreject);
 	    M_CHECKFORSTR(SAtemprejectSavCond);
 	    M_CHECKFORSTR(SAtemprejectsave);
 	    M_CHECKFORVAR(SAtemprejectoverwrite, "%d");
@@ -750,11 +756,28 @@ int local_scan(volatile int fd, uschar **return_text)
 	    M_CHECKFORSTR(SAmsgtemprej);
 	    M_CHECKFORSTR(SAmsgerror);
 
+
 	}
 	
 	readconffile=1;
     }
 
+#define M_CONDTOFLOAT(VAR) \
+    if ((expand=expand_string( VAR )) == NULL) \
+    { \
+	PANIC(string_sprintf(#VAR " config expansion failure on %s", #VAR ));\
+    } \
+    sscanf(expand, "%f", &VAR ## threshold); \
+    if (SAEximDebug > 2) \
+    { \
+	log_write(0, LOG_MAIN, "SA: Debug3: expanded " #VAR " = %.2f", VAR ## threshold); \
+    }\
+
+    M_CONDTOFLOAT(SAteergrube);
+    M_CONDTOFLOAT(SAdevnull);
+    M_CONDTOFLOAT(SApermreject);
+    M_CONDTOFLOAT(SAtempreject);
+
     /* Initialize the list of recipients here */
     rcptlist=string_copy(recipients_list[0].address);
     for (i=1; i < recipients_count && strlen((char *)rcptlist) < 7998 - strlen(recipients_list[i].address); i++)
@@ -1036,8 +1059,8 @@ restart:
            beginning of the loop without reading a new line since we already
 	   did that */
 	if (parsemlheader(buffer, (FILE *)readfh, "Subject", NULL)) goto restart;
-	if (parsemlheader(buffer, (FILE *)readfh, "Content-Type", NULL)) goto restart;
-	if (parsemlheader(buffer, (FILE *)readfh, "Content-Transfer-Encoding", NULL)) goto restart;
+	if ((SARewriteBody == 1) && parsemlheader(buffer, (FILE *)readfh, "Content-Type", NULL)) goto restart;
+	if ((SARewriteBody == 1) && parsemlheader(buffer, (FILE *)readfh, "Content-Transfer-Encoding", NULL)) goto restart;
 
 	if (parsemlheader(buffer, (FILE *)readfh, "X-Spam-Flag", &xspamflag))
 	{
@@ -1064,17 +1087,17 @@ restart:
 	     * for this mail, since it's been whitelisted */
 	    if (SAgreylistiswhitestr && strstr(xspamstatus, SAgreylistiswhitestr))
 	    {
-	        SAtempreject+=SAgreylistraisetempreject;
+	        SAtemprejectthreshold+=SAgreylistraisetempreject;
 		if (SAEximDebug > 2)
 		{
-		    log_write(0, LOG_MAIN, "SA: Debug3: read %s string, setting SAtempreject is now %f", SAgreylistiswhitestr, SAtempreject);
+		    log_write(0, LOG_MAIN, "SA: Debug3: read %s string, SAtempreject is now changed to %f", SAgreylistiswhitestr, SAtemprejectthreshold);
 		}
 	    }
 	    else
 	    {
 		if (SAEximDebug > 2)
 		{
-		    log_write(0, LOG_MAIN, "SA: Debug3: did not find read GREYLIST_ISWHITE strin in X-Spam-Status");
+		    log_write(0, LOG_MAIN, "SA: Debug3: did not find read GREYLIST_ISWHITE string in X-Spam-Status");
 		}
 	    }
 
@@ -1282,7 +1305,7 @@ restart:
 	    }
 	}
 
-	if (dorej && spamvalue >= SAteergrube)
+	if (dorej && spamvalue >= SAteergrubethreshold)
 	{
 	    doteergrube=1;
 	    if (SAteergrubecond[0] != '1' || SAteergrubecond[1] != 0)
@@ -1323,7 +1346,7 @@ restart:
 		CHECKERR(ret,where,line);
 	    }
 
-	    spamstatus=string_sprintf("%s trigger=%.1f", spamstatus, SAteergrube);
+	    spamstatus=string_sprintf("%s trigger=%.1f", spamstatus, SAteergrubethreshold);
 	    /* Exim might want to stop us if we run for too long, but that's
 	     * exactly what we're trying to do, so let's override that */
 	    alarm(0);
@@ -1358,27 +1381,27 @@ restart:
 	    *return_text=string_sprintf(SAmsgteergruberej, spamstatus);
 	    return LOCAL_SCAN_TEMPREJECT_NOLOGHDR;
 	}
-	else if (dorej && spamvalue >= SAdevnull)
+	else if (dorej && spamvalue >= SAdevnullthreshold)
 	{
 	    ret=savemail(fd, fdstart, SAdevnullsave, "SAdevnullsave", (char *)mesgfn, SAmaxarchivebody, SAdevnullSavCond);
 	    CHECKERR(ret,where,line);
 
 	    recipients_count=0;
-	    spamstatus=string_sprintf("%s trigger=%.1f", spamstatus, SAdevnull);
+	    spamstatus=string_sprintf("%s trigger=%.1f", spamstatus, SAdevnullthreshold);
 	    log_write(0, LOG_REJECT | LOG_MAIN, "SA: Action: silently tossed message: %s (scanned in %d/%d secs | Message-Id: %s). %s", spamstatus, scantime, fulltime, safemesgid, mailinfo);
 	    return LOCAL_SCAN_ACCEPT;
 	}
-	else if (dorej && spamvalue >= SApermreject)
+	else if (dorej && spamvalue >= SApermrejectthreshold)
 	{
 	    ret=savemail(fd, fdstart, SApermrejectsave, "SApermrejectsave", (char *)mesgfn, SAmaxarchivebody, SApermrejectSavCond);
 	    CHECKERR(ret,where,line);
 
-	    spamstatus=string_sprintf("%s trigger=%.1f", spamstatus, SApermreject);
+	    spamstatus=string_sprintf("%s trigger=%.1f", spamstatus, SApermrejectthreshold);
 	    log_write(0, LOG_MAIN | LOG_REJECT, "SA: Action: permanently rejected message: %s (scanned in %d/%d secs | Message-Id: %s). %s", spamstatus, scantime, fulltime, safemesgid, mailinfo);
 	    *return_text=string_sprintf(SAmsgpermrej, spamstatus);
 	    return LOCAL_SCAN_REJECT_NOLOGHDR;
 	}
-	else if (dorej && spamvalue >= SAtempreject)
+	else if (dorej && spamvalue >= SAtemprejectthreshold)
 	{
 	    /* Yeah, gotos are harmful, but that'd be a function with a lot
 	     * of options to send, so, here's a small shortcut */
@@ -1397,7 +1420,7 @@ restart:
 	/* This is an exception to the rule, for grey listing, we allow for
 	 * sending back a tempreject on SA scores that aren't considered as
 	 * spam (greylisting is now done directly in spamassassin though */
-	if (spamvalue >= SAtempreject)
+	if (spamvalue >= SAtemprejectthreshold)
 	{
 	    dotempreject:
 
@@ -1416,7 +1439,7 @@ restart:
 		CHECKERR(ret,where,line);
 	    }
 
-	    spamstatus=string_sprintf("%s trigger=%.1f", spamstatus, SAtempreject);
+	    spamstatus=string_sprintf("%s trigger=%.1f", spamstatus, SAtemprejectthreshold);
 	    log_write(0, LOG_MAIN | LOG_REJECT, "SA: Action: temporarily rejected message: %s (scanned in %d/%d secs | Message-Id: %s). %s", spamstatus, scantime, fulltime, safemesgid, mailinfo);
 	    *return_text=string_sprintf(SAmsgtemprej, spamstatus);
 	    return LOCAL_SCAN_TEMPREJECT_NOLOGHDR;
